module MyRepresentatives
  module Commonwealth
    class CSVUpper
      include MyRepresentatives::Abbreviatable
      include MyRepresentatives::Fileable
      include MyRepresentatives::Guessable

      attr_accessor :csv_url, :csv_filename, :people, :logger

      def initialize
        @logger       = Logger.new(STDOUT)
        @csv_url      = "http://www.aph.gov.au/~/media/03%20Senators%20and%20Members/Address%20Labels%20and%20CSV%20files/allsenel.csv?la=en"
        @csv_filename = "tmp/csv_commonwealth_upper.csv"
        @people       = []

        csv_from_url
        people_from_csv
      end

      private

        def csv_from_url
          create_tmp
          begin
            open(@csv_filename, "wb") do |file|
              open(@csv_url) do |uri|
                 file.write(uri.read)
              end
            end
          rescue SocketError => err
            @logger.debug("Unable to connect to #{@csv_url}")
            raise err
          end
        end

        def people_from_csv
          CSV.foreach(@csv_filename, headers: true) do |row|

            # Setup the electorate
            electorate_name  = state_abbreviation(find_electorate_state(row))
            electorate       = Electorate.new(electorate_name)
            electorate.commonwealth_upper!(electorate_name)

            # Setup the Representative (Person)
            person = Person.new(electorate)

            person.first_name                 = find_first_name(row)
            person.last_name                  = find_last_name(row)
            person.preferred_name             = find_preferred_name(row)
            person.title                      = find_title(row)
            person.honorifics                 = find_honorifics(row)
            person.phone                      = find_phone(row)
            person.party_name                 = find_party(row)
            person.salutation                 = find_salutation(row)
            person.gender                     = find_gender(row)
            person.formal_name                = format_formal_name(person)
            person.physical_address           = format_physical_address(row)
            person.postal_address             = format_postal_address(row)
            person.email                      = nil
            person.image_url                  = nil
            person.homepage_url               = nil

            @people << person
          end
        end

        def find_first_name(arr)
          arr["First Name"]
        end

        def find_last_name(arr)
          arr["Surname"]
        end

        def find_preferred_name(arr)
          arr["Preferred Name"]
        end

        def find_title(arr)
          arr["Title"]
        end

        def find_honorifics(arr)
          arr["Honorifics"]
        end

        def find_electorate(arr)
          arr["Electorate"]
        end

        def find_electorate_state(arr)
          arr["State"]
        end

        def find_gender(arr)
          arr["Gender"]
        end

        def find_party(arr)
          arr["Political Party"]
        end

        def find_phone(arr)
          arr["ElectorateTelephone"] ? arr["ElectorateTelephone"] : arr["Telephone"]
        end

        def find_salutation(arr)
          arr["Salutation"]
        end

        def format_formal_name(person)
          if person.preferred_name
            "#{person.title} #{person.preferred_name} #{person.last_name} #{person.honorifics}"
          else
            "#{person.title} #{person.first_name} #{person.last_name} #{person.honorifics}"
          end
        end

        def format_physical_address(arr)
          "#{arr["ElectorateAddressLine1"]}\n#{arr["ElectorateAddressLine2"]}\n#{arr["ElectorateSuburb"]} #{arr["ElectorateState"]} #{arr["ElectoratePostCode"]}"
        end

        def format_postal_address(arr)
          "#{arr["LabelAddress"]}\n#{arr["LabelSuburb"]} #{arr["LabelState"]} #{arr["LabelPostCode"]}"
        end

    end #CSVUpper
  end # Commonwealth
end # MyRepresentatives
