module MyRepresentatives
  module TAS
    class CSVUpper
      include MyRepresentatives::Fileable
      include MyRepresentatives::Guessable

      attr_accessor :csv_url, :csv_filename, :people

      def initialize
        # Tasmania's Parliament website has a word document for their House of Assembly and  Legislative Council.
        # At this time, this gem uses a csv manually created by https://github.com/tennantje/.
        # https://gist.github.com/tennantje/c662bd7a2ac44d2e5839ad73d6d0a80e
        # This gem has opted for this approach given that there is also no scrapable data on the Tasmanian Parliament website,
        # whereas Northern Territory (for example) has no CSV file, but does at least have a scrapable website.
        @csv_url      = "https://gist.githubusercontent.com/tennantje/c662bd7a2ac44d2e5839ad73d6d0a80e/raw"
        @csv_filename = "tmp/csv_tas_upper.csv"
        @people       = []

        csv_from_url
        people_from_csv
      end

      private

        def csv_from_url
          create_tmp
          begin
            open(@csv_filename, "wb") do |file|
              open(@csv_url) do |uri|
                 file.write(uri.read)
              end
            end
          rescue SocketError => err
            @logger.debug("Unable to connect to #{@csv_url}")
            raise err
          end
        end

        def people_from_csv
          CSV.foreach(@csv_filename, headers: true) do |row|

            # Setup the electorate
            electorate_name = find_electorate(row)
            electorate      = Electorate.new(electorate_name)
            electorate.state_tas_upper!

            # Setup the Representative (Person)
            person = Person.new(electorate)

            person.title                      = find_title(row)
            person.first_name                 = find_first_name(row)
            person.preferred_name             = find_preferred_name(row)
            person.last_name                  = find_last_name(row)
            person.email                      = find_email(row)
            person.phone                      = find_phone(row)
            person.party_name                 = find_party(row)
            person.gender                     = find_gender(row)
            person.honorifics                 = "MLC"
            person.formal_name                = format_formal_name(person)
            person.physical_address           = format_physical_address(row)
            person.postal_address             = format_postal_address(row)
            person.salutation                 = nil
            person.image_url                  = nil
            person.homepage_url               = nil

            @people << person

          end
        end

        def find_first_name(arr)
          arr["FirstName"]
        end

        def find_last_name(arr)
          arr["LastName"]
        end

        def find_preferred_name(arr)
          arr["PreferredName"]
        end

        def find_title(arr)
          arr["Title"]
        end

        def find_electorate(arr)
          arr["Electorate"]
        end

        def find_email(arr)
          arr["Email"]
        end

        def find_party(arr)
          arr["Party"]
        end

        def find_phone(arr)
          arr["Phone"]
        end

        def find_gender(arr)
          arr["Gender"]
        end

        def format_formal_name(person)
          if person.preferred_name
            "#{person.title} #{person.preferred_name} #{person.last_name} #{person.honorifics}"
          else
            "#{person.title} #{person.first_name} #{person.last_name} #{person.honorifics}"
          end
        end

        def format_physical_address(arr)
          "#{arr["Address1"]}\n#{arr["City"]}\nTAS #{arr["Postcode"]}"
        end

        def format_postal_address(arr)
          "#{arr["Address1"]}\n#{arr["City"]}\nTAS #{arr["Postcode"]}"
        end

    end # CSVUpper
  end # TAS
end # MyRepresentatives
